# BambangShop Receiver App
Tutorial and Example for Advanced Programming 2025 - Faculty of Computer Science, Universitas Indonesia

---

## About this Project
In this repository, we have provided you a REST (REpresentational State Transfer) API project using Rocket web framework.

This project consists of four modules:
1.  `controller`: this module contains handler functions used to receive request and send responses.
    In Model-View-Controller (MVC) pattern, this is the Controller part.
2.  `model`: this module contains structs that serve as data containers.
    In MVC pattern, this is the Model part.
3.  `service`: this module contains structs with business logic methods.
    In MVC pattern, this is also the Model part.
4.  `repository`: this module contains structs that serve as databases.
    You can use methods of the struct to get list of objects, or operating an object (create, read, update, delete).

This repository provides a Rocket web framework skeleton that you can work with.

As this is an Observer Design Pattern tutorial repository, you need to implement a feature: `Notification`.
This feature will receive notifications of creation, promotion, and deletion of a product, when this receiver instance is subscribed to a certain product type.
The notification will be sent using HTTP POST request, so you need to make the receiver endpoint in this project.

## API Documentations

You can download the Postman Collection JSON here: https://ristek.link/AdvProgWeek7Postman

After you download the Postman Collection, you can try the endpoints inside "BambangShop Receiver" folder.

Postman is an installable client that you can use to test web endpoints using HTTP request.
You can also make automated functional testing scripts for REST API projects using this client.
You can install Postman via this website: https://www.postman.com/downloads/

## How to Run in Development Environment
1.  Set up environment variables first by creating `.env` file.
    Here is the example of `.env` file:
    ```bash
    ROCKET_PORT=8001
    APP_INSTANCE_ROOT_URL=http://localhost:${ROCKET_PORT}
    APP_PUBLISHER_ROOT_URL=http://localhost:8000
    APP_INSTANCE_NAME=Safira Sudrajat
    ```
    Here are the details of each environment variable:
    | variable                | type   | description                                                     |
    |-------------------------|--------|-----------------------------------------------------------------|
    | ROCKET_PORT             | string | Port number that will be listened by this receiver instance.    |
    | APP_INSTANCE_ROOT_URL   | string | URL address where this receiver instance can be accessed.       |
    | APP_PUUBLISHER_ROOT_URL | string | URL address where the publisher instance can be accessed.       |
    | APP_INSTANCE_NAME       | string | Name of this receiver instance, will be shown on notifications. |
2.  Use `cargo run` to run this app.
    (You might want to use `cargo check` if you only need to verify your work without running the app.)
3.  To simulate multiple instances of BambangShop Receiver (as the tutorial mandates you to do so),
    you can open new terminal, then edit `ROCKET_PORT` in `.env` file, then execute another `cargo run`.

    For example, if you want to run 3 (three) instances of BambangShop Receiver at port `8001`, `8002`, and `8003`, you can do these steps:
    -   Edit `ROCKET_PORT` in `.env` to `8001`, then execute `cargo run`.
    -   Open new terminal, edit `ROCKET_PORT` in `.env` to `8002`, then execute `cargo run`.
    -   Open another new terminal, edit `ROCKET_PORT` in `.env` to `8003`, then execute `cargo run`.

## Mandatory Checklists (Subscriber)
-   [x] Clone https://gitlab.com/ichlaffterlalu/bambangshop-receiver to a new repository.
-   **STAGE 1: Implement models and repositories**
    -   [x] Commit: `Create Notification model struct.`
    -   [x] Commit: `Create SubscriberRequest model struct.`
    -   [x] Commit: `Create Notification database and Notification repository struct skeleton.`
    -   [x] Commit: `Implement add function in Notification repository.`
    -   [x] Commit: `Implement list_all_as_string function in Notification repository.`
    -   [x] Write answers of your learning module's "Reflection Subscriber-1" questions in this README.
-   **STAGE 2: Implement services and controllers**
    -   [x] Commit: `Create Notification service struct skeleton.`
    -   [x] Commit: `Implement subscribe function in Notification service.`
    -   [x] Commit: `Implement subscribe function in Notification controller.`
    -   [x] Commit: `Implement unsubscribe function in Notification service.`
    -   [x] Commit: `Implement unsubscribe function in Notification controller.`
    -   [x] Commit: `Implement receive_notification function in Notification service.`
    -   [x] Commit: `Implement receive function in Notification controller.`
    -   [x] Commit: `Implement list_messages function in Notification service.`
    -   [x] Commit: `Implement list function in Notification controller.`
    -   [x] Write answers of your learning module's "Reflection Subscriber-2" questions in this README.

## Your Reflections
This is the place for you to write reflections:

### Mandatory (Subscriber) Reflections

#### Reflection Subscriber-1
> In this tutorial, we used RwLock<> to synchronise the use of Vec of Notifications. Explain why it is necessary for this case, and explain why we do not use Mutex<> instead?

1. Dalam tutorial ini kita menggunakan `RwLock<>` untuk menyinkronisasi vektor `Notification` karena ia memungkinkan banyak thread membaca data secara bersamaan, namun hanya satu thread yang dapat menulis pada satu waktu. Ini sangat efisien karena dalam aplikasi web kita, operasi pembacaan notifikasi jauh lebih sering terjadi dibanding penulisan. Jika menggunakan `Mutex<>`, hanya satu thread yang dapat mengakses data pada satu waktu, baik untuk membaca maupun menulis, yang akan menciptakan bottleneck yang tidak perlu. Dengan `RwLock<>`, thread yang hanya membaca tidak akan terblokir ketika tidak ada operasi penulisan, sehingga meningkatkan konkurensi dan performa aplikasi.

> In this tutorial, we used lazy_static external library to define Vec and DashMap as a “static” variable. Compared to Java where we can mutate the content of a static variable via a static function, why did not Rust allow us to do so?

2. Rust tidak mengizinkan modifikasi langsung pada variabel static karena prinsip keamanan yang menjadi inti dari bahasa ini. Tidak seperti Java, Rust memiliki aturan ketat tentang mutabilitas untuk mencegah race condition dalam lingkungan multi-threaded. Variabel static yang bersifat global dan dapat diakses dari mana saja sehingga memungkinkan modifikasi langsung akan bertentangan dengan sistem kepemilikan dan peminjaman Rust. Oleh karena itu, kita menggunakan library lazy_static bersama dengan RwLock untuk membuat variabel static yang bisa diubah dengan aman, dengan memastikan hanya satu thread yang bisa melakukan perubahan pada satu waktu.


#### Reflection Subscriber-2
> Have you explored things outside of the steps in the tutorial, for example: src/lib.rs? If not, explain why you did not do so. If yes, explain things that you have learned from those other parts of code.

1.Saya mencoba memahami isi dari src/lib.rs dan menemukan bahwa file ini berperan penting dalam konfigurasi aplikasi. Salah satu hal menarik yang saya pelajari adalah bagaimana lazy_static! digunakan untuk membuat REQWEST_CLIENT dan APP_CONFIG sebagai variabel statis yang diinisialisasi sekali dan dapat digunakan di seluruh aplikasi. Selain itu, AppConfig mengatur konfigurasi aplikasi dengan nilai default, tetapi bisa diubah menggunakan variabel lingkungan (.env) melalui Figment. Saya juga memperhatikan bahwa ada tipe Result<T, E> yang sudah dikustomisasi untuk menangani error dengan format JSON, yang membuat penanganan error menjadi lebih terstruktur.

> Since you have completed the tutorial by now and have tried to test your notification system by spawning multiple instances of Receiver, explain how Observer pattern eases you to plug in more subscribers. How about spawning more than one instance of Main app, will it still be easy enough to add to the system?

2. Dengan pola Observer, menambahkan instance Subscriber baru sangat mudah—cukup dengan mengatur port berbeda tanpa konfigurasi tambahan. Namun, menambahkan instance baru dari Main app memerlukan lebih banyak langkah, seperti menyesuaikan konfigurasi di Receiver dan memastikan setiap Subscriber melakukan subscribe secara manual ke Main app yang baru. Untuk menyederhanakan proses ini, implementasi subscribe dan unsubscribe di Receiver perlu diperbarui agar dapat berkomunikasi dengan semua instance Main app.

> Have you tried to make your own Tests, or enhance documentation on your Postman collection? If you have tried those features, tell us whether it is useful for your work (it can be your tutorial work or your Group Project).

3. Saya belum pernah mencoba membuat tes sendiri di Postman, tetapi dari koleksi tes yang tersedia, fitur ini tampaknya sangat berguna. Dengan Postman, saya dapat menguji API tanpa harus menulis kode pengujian secara manual.
